'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ExifReader = _interopDefault(require('exifreader'));
var sizeOf = _interopDefault(require('image-size'));

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);
  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }
  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var ImageRegion = /*#__PURE__*/function () {
  function ImageRegion() {
    /**
     * Identifier for the region. Unique within the image.
     */
    this.id = '';
    /**
     * Names for the region, possibly in multiple languages.
     */
    this.names = [];
    /**
     * Region shape, can be `rectangle`, `circle` or `polygon`.
     */
    this.shape = '';
    /**
     * Unit used for `x`, `y`, `width`, `height`, `radius` and `vertices`. Can be
     * `relative` or `pixel`, see
     * https://iptc.org/std/photometadata/specification/IPTC-PhotoMetadata#boundary-measuring-unit
     */
    this.unit = '';
  }
  /**
   * @return `true` if the region matches the given shape and role filters.
   */
  var _proto = ImageRegion.prototype;
  _proto.matches = function matches(shapeFilter, roleFilter) {
    return this._matchesShapeFilter(shapeFilter) && this._matchesRoleFilter(roleFilter);
  };
  _proto._matchesShapeFilter = function _matchesShapeFilter(shapeFilter) {
    return shapeFilter === 'any' || this.shape === shapeFilter;
  };
  _proto._matchesRoleFilter = function _matchesRoleFilter(roleFilter) {
    if (roleFilter === 'any') {
      return true;
    }
    if (roleFilter === 'crop') {
      for (var _iterator = _createForOfIteratorHelperLoose(this.roles || []), _step; !(_step = _iterator()).done;) {
        var role = _step.value;
        if (ImageRegion._CROP_XML_ROLES.includes(role)) {
          return true;
        }
      }
    }
    return false;
  };
  return ImageRegion;
}();
/**
 * See https://cv.iptc.org/newscodes/imageregionrole/
 */
ImageRegion._CROP_XML_ROLES = ['cropping', 'recommended cropping', 'landscape format cropping', 'portrait format cropping', 'square format cropping', 'http://cv.iptc.org/newscodes/imageregionrole/cropping', 'http://cv.iptc.org/newscodes/imageregionrole/recomCropping', 'http://cv.iptc.org/newscodes/imageregionrole/landscapeCropping', 'http://cv.iptc.org/newscodes/imageregionrole/portraitCropping', 'http://cv.iptc.org/newscodes/imageregionrole/squareCropping'];

/**
 * Parses the XMP metadata of an image, relevant for Image Display Control, i.e.
 * mostly the image regions, see
 * https://iptc.org/std/photometadata/specification/IPTC-PhotoMetadata#image-region
 */
var Parser = /*#__PURE__*/function () {
  /**
   * @param buffer The image file content.
   */
  function Parser(buffer) {
    this._size = null;
    this._buffer = buffer;
    try {
      var metadata = ExifReader.load(this._buffer, {
        expanded: true
      });
      this._xmpMetadata = metadata.xmp || {};
    } catch (e) {
      this._xmpMetadata = {};
    }
  }
  /**
   * Returns XMP IDC metadata in a format similar to what this web-component
   * expects: https://github.com/Frameright/image-display-control-web-component
   *
   * @param shapeFilter Can be used to retrieve only regions of a specific
   *                    shape, e.g. 'rectangle'.
   * @param roleFilter Can be used to retrieve only regions of a specific kind
   *                   of role, e.g. 'crop'.
   * @param essentialOnly If true, only essential region properties will be
   *                      returned, e.g. properties like `types` and `roles`
   *                      will be skipped.
   */
  var _proto = Parser.prototype;
  _proto.getIdcMetadata = function getIdcMetadata(shapeFilter, roleFilter, essentialOnly) {
    var _this$_xmpMetadata$Im,
      _this = this;
    if (shapeFilter === void 0) {
      shapeFilter = 'any';
    }
    if (roleFilter === void 0) {
      roleFilter = 'any';
    }
    if (essentialOnly === void 0) {
      essentialOnly = true;
    }
    var result = [];
    if (!Array.isArray((_this$_xmpMetadata$Im = this._xmpMetadata.ImageRegion) == null ? void 0 : _this$_xmpMetadata$Im.value)) {
      return result;
    }
    var xmpRegions = this._xmpMetadata.ImageRegion.value;
    xmpRegions.forEach(function (xmpRegion) {
      var region = _this._xmpRegionToImageRegion(xmpRegion);
      if (region.matches(shapeFilter, roleFilter)) {
        if (essentialOnly) {
          delete region.types;
          delete region.roles;
          if (region.unit === 'relative') {
            delete region.imageWidth;
            delete region.imageHeight;
          }
          switch (region.shape) {
            case 'rectangle':
              delete region.radius;
              delete region.vertices;
              break;
            case 'circle':
              delete region.width;
              delete region.height;
              delete region.vertices;
              break;
            case 'polygon':
              delete region.x;
              delete region.y;
              delete region.width;
              delete region.height;
              delete region.radius;
              break;
          }
        }
        result.push(region);
      }
    });
    return result;
  }
  /**
   * Dumps all XMP metadata.
   *
   * @return An instance of
   *         [ExifReader.XmpTags](https://github.com/mattiasw/ExifReader/blob/main/exif-reader.d.ts#L121)
   */;
  _proto.getXmpMetadata = function getXmpMetadata() {
    return this._xmpMetadata;
  }
  /**
   * Returns the size of the image in pixels.
   *
   * @note Caches the result in `this._size` for future calls.
   */;
  _proto.getSize = function getSize() {
    if (this._size) {
      return this._size;
    }
    var size = sizeOf(this._buffer);
    this._size = {
      width: size.width || 0,
      height: size.height || 0
    };
    return this._size;
  }
  // Converts a bag of entity or concepts to an array of strings. See
  // https://iptc.org/std/photometadata/specification/IPTC-PhotoMetadata#entity-or-concept-structure
  ;
  Parser._xmpEntityOrConceptsToStringArray = function _xmpEntityOrConceptsToStringArray(xmpEntityOrConcepts) {
    var result = [];
    var bag = xmpEntityOrConcepts.value;
    if (Array.isArray(bag)) {
      bag.forEach(function (xmpEntityOrConcept) {
        result.push.apply(result, Parser._xmpEntityOrConceptToStringArray(xmpEntityOrConcept));
      });
    } else if ('Bag' in bag) {
      // happens when there is only one element in the bag
      result.push.apply(result, Parser._xmpEntityOrConceptToStringArray(bag['Bag'].value));
    }
    return result;
  }
  // Converts an entity or concept to an array of strings. See
  // https://iptc.org/std/photometadata/specification/IPTC-PhotoMetadata#entity-or-concept-structure
  ;
  Parser._xmpEntityOrConceptToStringArray = function _xmpEntityOrConceptToStringArray(xmpEntityOrConcept) {
    var result = [];
    if ('Name' in xmpEntityOrConcept) {
      result.push.apply(result, Parser._xmpAltOrBagToStringArray(xmpEntityOrConcept['Name']));
    }
    if ('Identifier' in xmpEntityOrConcept) {
      result.push.apply(result, Parser._xmpAltOrBagToStringArray(xmpEntityOrConcept['Identifier']));
    }
    return result;
  }
  // Converts an XMP rdf:Alt or rdf:Bag tag to an array of strings.
  ;
  Parser._xmpAltOrBagToStringArray = function _xmpAltOrBagToStringArray(xmpAltOrBag) {
    var result = [];
    if (Array.isArray(xmpAltOrBag.value)) {
      xmpAltOrBag.value.forEach(function (item) {
        if (typeof item.value === 'string') {
          result.push(item.value);
        }
      });
    }
    return result;
  };
  Parser._xmpStringToNumber = function _xmpStringToNumber(xmpString) {
    if (typeof xmpString.value === 'string') {
      return parseFloat(xmpString.value);
    }
    return undefined;
  }
  // Converts an ImageRegion XMP tag to an ImageRegion object.
  ;
  _proto._xmpRegionToImageRegion = function _xmpRegionToImageRegion(region) {
    var result = new ImageRegion();
    var xmpId = 'rId' in region ? region['rId'].value : '';
    if (typeof xmpId === 'string') {
      result.id = xmpId;
    }
    if ('Name' in region) {
      result.names = Parser._xmpAltOrBagToStringArray(region['Name']);
    }
    if ('rCtype' in region) {
      result.types = Parser._xmpEntityOrConceptsToStringArray(region['rCtype']);
    }
    if ('rRole' in region) {
      result.roles = Parser._xmpEntityOrConceptsToStringArray(region['rRole']);
    }
    if ('RegionBoundary' in region) {
      var xmpRegionBoundary = region['RegionBoundary'].value;
      if ('rbShape' in xmpRegionBoundary) {
        var xmpShape = xmpRegionBoundary['rbShape'].value;
        if (typeof xmpShape === 'string') {
          result.shape = xmpShape;
        }
      }
      if ('rbUnit' in xmpRegionBoundary) {
        var xmpUnit = xmpRegionBoundary['rbUnit'].value;
        if (typeof xmpUnit === 'string') {
          result.unit = xmpUnit;
        }
      }
      if ('rbX' in xmpRegionBoundary) {
        result.x = Parser._xmpStringToNumber(xmpRegionBoundary['rbX']);
      }
      if ('rbY' in xmpRegionBoundary) {
        result.y = Parser._xmpStringToNumber(xmpRegionBoundary['rbY']);
      }
      if ('rbW' in xmpRegionBoundary) {
        result.width = Parser._xmpStringToNumber(xmpRegionBoundary['rbW']);
      }
      if ('rbH' in xmpRegionBoundary) {
        result.height = Parser._xmpStringToNumber(xmpRegionBoundary['rbH']);
      }
      if ('rbRx' in xmpRegionBoundary) {
        result.radius = Parser._xmpStringToNumber(xmpRegionBoundary['rbRx']);
      }
      if ('rbVertices' in xmpRegionBoundary) {
        var xmpVertices = xmpRegionBoundary['rbVertices'].value;
        if (Array.isArray(xmpVertices)) {
          xmpVertices.forEach(function (vertex) {
            var vertexX;
            var vertexY;
            if ('rbX' in vertex) {
              vertexX = Parser._xmpStringToNumber(vertex['rbX']);
            }
            if ('rbY' in vertex) {
              vertexY = Parser._xmpStringToNumber(vertex['rbY']);
            }
            if (vertexX !== undefined && vertexY !== undefined) {
              if (!Array.isArray(result.vertices)) {
                result.vertices = [];
              }
              result.vertices.push({
                x: vertexX,
                y: vertexY
              });
            }
          });
        }
      }
    }
    var size = this.getSize();
    result.imageWidth = size.width;
    result.imageHeight = size.height;
    return result;
  };
  return Parser;
}();

exports.ImageRegion = ImageRegion;
exports.Parser = Parser;
//# sourceMappingURL=image-display-control-metadata-parser.cjs.development.js.map
